<?php

namespace Drupal\syncloud\Service;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\ConnectException;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Exception\RequestException;

/**
 * Class Connector.
 */
class Queue {

  /**
   * Init funciton.
   */
  public function __construct() {
  }

  /**
   * Hook.
   */
  public function guzzlePushQueue() {
    $host = \Drupal::request()->getSchemeAndHttpHost();
    $client = new Client([
      'base_uri' => $host,
      'timeout' => 1,
    ]);
    try {
      $response = $client->get("$host/syncloud/queue");
      $message = $response->getStatusCode();
    }
    catch (ConnectException $e) {
      $message = 'ConnectException ' . $e->getMessage();
    }
    catch (RequestException $e) {
      $message = 'RequestException ' . $e->getMessage();
    }
    catch (ClientException $e) {
      $message = 'ClientException ' . $e->getMessage();
    }
    catch (GuzzleException $e) {
      $message = 'GuzzleException ' . $e->getMessage();
    }
    return $message;
  }

  /**
   * Push entity to queue.
   */
  public function queuePush(int $id, $syn_id) {
    $queue = \Drupal::queue('syncloud_queue');
    $queue->createQueue();
    $queue->createItem([
      'id' => $id,
      'syn_id' => $syn_id,
    ]);
  }

  /**
   * Process queue.
   */
  public function queueProcess() {
    $end = time() + 30;
    $queue = \Drupal::queue('syncloud_queue');
    while (time() < $end && ($item = $queue->claimItem())) {
      $syn = \Drupal::entityTypeManager()->getStorage('syn')->load($item->data['syn_id']);
      $storage = \Drupal::entityTypeManager()->getStorage($syn->type->value);
      $entity = $storage->load($item->data['id']);
      $config = \Drupal::configFactory()->get('syncloud.settings');
      if ($config->get('telegram-int')) {
        $uuid = \Drupal::state()->get('syncloud.uuid');
        $extra = $syn->extra->getValue()[0];
        $msg = [
          'ip' => $syn->ip->value,
          'type' => $syn->type->value,
          'host' => $syn->host->value,
          'mode' => $syn->mode->value,
          'matomo' => $syn->syn_id->value,
          'url' => $syn->url->value,
          'yandex' => $extra['yandex'],
          'google' => $extra['google'],
        ];
        if ($entity->getEntityTypeId() == 'commerce_order') {
          // Корзина закомплитилась.
          /** @var \Drupal\commerce_order\Entity\OrderInterface $entity */

          // Контактная информация.
          $profile = $entity->getBillingProfile();
          $profile_data = $profile->toArray() ?? [];
          foreach ($profile_data as $key => $value) {
            if (strpos($key, "field_customer_") !== FALSE && isset($value[0]['value'])) {
              $msg[$profile_data['uid'][0]['target_id']][str_replace("field_customer_", "commerce--field_", $key)] = [
                'label' => $profile->getFieldDefinition($key)->getLabel(),
                'value' => $value[0]['value'],
              ];
            }
          }

          // Элементы заказа.
          foreach ($entity->getItems() ?? [] as $order_item) {
            $currency = $order_item->getTotalPrice()->getCurrencyCode();
            $title = $order_item->getTitle();
            $msg[$title]['commerce--field_title'] = $order_item->getTitle();
            $msg[$title]['commerce--field_price'] = round($order_item->getTotalPrice()->getNumber(), 2) . " $currency";
            $msg[$title]['commerce--field_quantity'] = round($order_item->getQuantity(), 0);
            $msg[$title]['commerce--field_adjusted_price'] = round($order_item->getAdjustedTotalPrice()->getNumber(), 2) . " $currency";
          }

          // Сам заказ.
          $order_data = $entity->toArray();
          $msg['order']['commerce--field_order_number'] = $order_data['order_number'][0]['value'];
          if (isset($order_data['shipments']) && !empty($order_data['shipments'][0])) {
            $shipment_storage = \Drupal::entityTypeManager()->getStorage('commerce_shipment');
            $shipment = $shipment_storage->load($order_data['shipments'][0]['target_id']);
            $msg['total']['commerce--field_shipping_method'] = $shipment ? $shipment->getShippingMethod()->getName() : '';
          }
          if ($entity->hasField('payment_gateway') && $gateway = $entity->payment_gateway->entity) {
            $conf = $gateway->getPluginConfiguration();
            if ($gateway->id() !== 'cash') {
              $msg['total']['commerce--field_payment_gateway'] = "{$conf['display_label']}: {$conf['gateway']}";
            }
            else {
              $msg['total']['commerce--field_payment_gateway'] = $conf['display_label'];
            }
            $msg['total']['commerce--field_total_price'] = round($entity->getTotalPrice()->getNumber(), 2) . " $currency";
          }
          \Drupal::moduleHandler()->alter('syncloud_queue_preprocess_commerce', $entity, $syn->syn_id->value);
        }
        elseif ($entity->getEntityTypeId() == 'webform_submission') {
          // Вебформа.
          $msg['type'] = 'webform_submission';
          $webform = $entity->getWebform();
          foreach ($entity->getData() ?? [] as $key => $val) {
            $field = $webform->getElementDecoded($key);
            switch ($field['#type']) {
              case 'radios':
                $value = $field['#options'][$val];
                break;

              case 'checkbox':
                $value = $val == 1 ? 'Да' : 'Нет';
                break;

              default:
                $value = $val;
                break;
            }
            $msg["message--field_$key"] = [
              'label' => $field['#title'],
              'value' => $value,
            ];
          }
          $msg["message--field_webform_id"] = [
            'label' => 'id вебформы',
            'value' => $entity->bundle(),
          ];
          $msg["message--field_webform_title"] = [
            'label' => 'Название вебформы',
            'value' => $webform->get('title'),
          ];
          \Drupal::moduleHandler()->alter('syncloud_queue_preprocess_webform', $entity, $syn->syn_id->value);
        }
        else {
          // Контактная форма.
          $contact_form = $entity->getContactForm();
          $field_definitions = \Drupal::service('entity_field.manager')
            ->getFieldDefinitions('contact_message', $contact_form->id());
          $contact_form_field_skip = ['uuid', 'langcode', 'message', 'copy', 'recipient', 'created', 'uid', 'metatag', 'ip_address', 'contact_form', 'id'];
          $file_url_generator = \Drupal::service('file_url_generator');
          foreach ($entity->getFields() ?? [] as $key => $val) {
            if (in_array($key, $contact_form_field_skip) || empty($val)) {
              continue;
            }
            $field_entity = $val->entity;
            switch ($field_definitions[$key]->getType()) {
              case "entity_reference":
                $value = $field_entity->toUrl()->setAbsolute()->toString();
                break;

              case "file":
                $value = $file_url_generator->generateAbsoluteString($field_entity->getFileUri());
                break;

              default:
                $value = $val->getString();
                break;
            }

            $msg["message--field_$key"] = [
              'label' => $field_definitions[$key]->getLabel(),
              'value' => $value,
            ];
          }
          \Drupal::moduleHandler()->alter('syncloud_queue_preprocess_contactform', $entity, $syn->syn_id->value);
        }
        \Drupal::service('syncloud.mqtt')->run()->publish("\$telega/syncloud/$uuid/event/contact-message", json_encode($msg));
      }
      $queue->deleteItem($item);
    }
  }

}
